-- Declare cursor variables for the table structure
DECLARE @TABLE_SCHEMA								NVARCHAR(128)
DECLARE @TABLE_NAME									NVARCHAR(128)
DECLARE @COLUMN_NAME								NVARCHAR(128)
DECLARE @DATA_TYPE										NVARCHAR(128)
DECLARE @CHARACTER_MAXIMUM_LENGTH			NVARCHAR(128)
DECLARE @NUMERIC_PRECISION						NVARCHAR(128)
DECLARE @NUMERIC_SCALE								NVARCHAR(128)
DECLARE @IS_NULLABLE									NVARCHAR(128)
DECLARE @PRIMARY_KEY									NVARCHAR(128)

DECLARE @SQL_COMMAND								NVARCHAR(2048)
DECLARE @LAST_TABLE_NAME							NVARCHAR(128)
DECLARE @NewLineChar AS CHAR(2) = CHAR(13) + CHAR(10)

-- Create a cursor to scan through all the fields in each table
-- Note each table can only have a single column as a primary key
-- all tables are on the same schema


PRINT 'CREATE SCHEMA Access'
PRINT 'GO'
PRINT @NewLineChar
PRINT @NewLineChar
	
DECLARE tc_cursor CURSOR FOR
SELECT	t.TABLE_SCHEMA,
			t.TABLE_NAME,
			c.COLUMN_NAME,
			DATA_TYPE,
			CHARACTER_MAXIMUM_LENGTH,
			NUMERIC_PRECISION,
			NUMERIC_SCALE,
			IS_NULLABLE,
			CASE WHEN cu.COLUMN_NAME IS NULL THEN 'N' ELSE 'Y' END AS PRIMARY_KEY	
FROM INFORMATION_SCHEMA.COLUMNS c
	INNER JOIN INFORMATION_SCHEMA.TABLES t
		ON c.TABLE_SCHEMA = t.TABLE_SCHEMA
		AND c.TABLE_NAME = t.TABLE_NAME
	LEFT OUTER JOIN INFORMATION_SCHEMA.TABLE_CONSTRAINTS tc
		ON t.TABLE_SCHEMA = tc.TABLE_SCHEMA
		AND t.TABLE_NAME = tc.TABLE_NAME
	LEFT OUTER JOIN INFORMATION_SCHEMA.CONSTRAINT_COLUMN_USAGE cu
		ON cu.CONSTRAINT_NAME = tc.CONSTRAINT_NAME
		AND cu.COLUMN_NAME = c.COLUMN_NAME
WHERE t.TABLE_SCHEMA = 'Access' AND t.TABLE_TYPE = 'BASE TABLE'
AND tc.CONSTRAINT_TYPE = 'primary Key'
ORDER BY t.TABLE_SCHEMA,t.TABLE_NAME,ORDINAL_POSITION

SET @LAST_TABLE_NAME = ''
SET @SQL_COMMAND = ''
OPEN tc_cursor   
FETCH NEXT FROM tc_cursor INTO @TABLE_SCHEMA, 
												@TABLE_NAME,
												@COLUMN_NAME,  
												@DATA_TYPE,
												@CHARACTER_MAXIMUM_LENGTH,
												@NUMERIC_PRECISION,
												@NUMERIC_SCALE,
												@IS_NULLABLE,
												@PRIMARY_KEY

WHILE @@FETCH_STATUS = 0   
BEGIN   
	IF @LAST_TABLE_NAME <> @TABLE_NAME
	BEGIN
		IF @SQL_COMMAND <> ''
		BEGIN
			SET @SQL_COMMAND = @SQL_COMMAND + @NewLineChar + ')' + @NewLineChar
			PRINT @SQL_COMMAND
			PRINT 'GO'
			PRINT @NewLineChar
			PRINT @NewLineChar
			SET @SQL_COMMAND = ''
		END
		SET @LAST_TABLE_NAME = @TABLE_NAME
		SET @SQL_COMMAND ='CREATE TABLE ' + @TABLE_SCHEMA + '.[' + @TABLE_NAME + '] ('
	END
	ELSE
	BEGIN
		SET @SQL_COMMAND  = @SQL_COMMAND + ','
	END

	-- Build the logic for each field
	SET @SQL_COMMAND = @SQL_COMMAND + @NewLineChar
	SET @SQL_COMMAND = @SQL_COMMAND + @COLUMN_NAME + ' ' + @DATA_TYPE
	
	IF @IS_NULLABLE = 'NO'
	BEGIN
		SET @SQL_COMMAND = @SQL_COMMAND + ' NOT NULL '
	END
	
	IF @PRIMARY_KEY = 'Y'
	BEGIN
		SET @SQL_COMMAND = @SQL_COMMAND + ' PRIMARY KEY '
	END
	
	IF @CHARACTER_MAXIMUM_LENGTH IS NOT NULL
	BEGIN
		IF @CHARACTER_MAXIMUM_LENGTH = '-1'
		BEGIN
			SET @SQL_COMMAND = @SQL_COMMAND + ' (MAX)'
		END
		ELSE
		BEGIN
			SET @SQL_COMMAND = @SQL_COMMAND + ' ( ' + @CHARACTER_MAXIMUM_LENGTH + ')'
		END
	END
	
	IF @DATA_TYPE = 'decimal' 
	BEGIN
		SET @SQL_COMMAND = @SQL_COMMAND + ' ( ' + @NUMERIC_PRECISION + ',' + @NUMERIC_SCALE + ')'
	END
	
	FETCH NEXT FROM tc_cursor INTO @TABLE_SCHEMA, 
												@TABLE_NAME,
												@COLUMN_NAME,  
												@DATA_TYPE,
												@CHARACTER_MAXIMUM_LENGTH,
												@NUMERIC_PRECISION,
												@NUMERIC_SCALE,
												@IS_NULLABLE,
												@PRIMARY_KEY 
END   

IF @TABLE_NAME <> ''
BEGIN
	SET @SQL_COMMAND = @SQL_COMMAND + ' )' + @NewLineChar
END

IF @SQL_COMMAND <> ''
BEGIN
	PRINT @SQL_COMMAND
	PRINT 'GO'
	PRINT @NewLineChar
	PRINT @NewLineChar
END
	
CLOSE tc_cursor   
DEALLOCATE tc_cursor

-- SOMETHING IS ROUND THE WRONG WAY IN THE NAME OF FK AND RK

-- Declare cursor variables for the table structure
-- Cursor scans through the relationships, to re-create them

DECLARE @PRIMARYTABLE								NVARCHAR(128)
DECLARE @FOREIGNKEY									NVARCHAR(128)
DECLARE @RELATEDTABLE								NVARCHAR(128)
DECLARE @RELATEDKEY									NVARCHAR(128)

-- Next Build the relationships
DECLARE tc_cursor CURSOR FOR
SELECT 
		fk.TABLE_SCHEMA,
		fk.TABLE_NAME AS PRIMARYTABLE,
        cu.COLUMN_NAME AS FOREIGNKEY, 
        pk.TABLE_NAME AS RELATEDTABLE, 
        Q1.COLUMN_NAME As RELATEDKEY
    FROM 
        INFORMATION_SCHEMA.REFERENTIAL_CONSTRAINTS c 
        INNER JOIN 
			INFORMATION_SCHEMA.TABLE_CONSTRAINTS fk 
				ON c.CONSTRAINT_NAME = fk.CONSTRAINT_NAME 
        INNER JOIN 
			INFORMATION_SCHEMA.TABLE_CONSTRAINTS pk 
				ON c.UNIQUE_CONSTRAINT_NAME = pk.CONSTRAINT_NAME 
        INNER JOIN 
			INFORMATION_SCHEMA.KEY_COLUMN_USAGE cu 
				ON c.CONSTRAINT_NAME = cu.CONSTRAINT_NAME 
        INNER JOIN 
        ( 
            SELECT 
                t2.TABLE_NAME, c2.COLUMN_NAME 
            FROM 
                INFORMATION_SCHEMA.TABLE_CONSTRAINTS t2
                INNER JOIN 
					INFORMATION_SCHEMA.KEY_COLUMN_USAGE c2 
						ON t2.CONSTRAINT_NAME = c2.CONSTRAINT_NAME 
                WHERE t2.CONSTRAINT_TYPE = 'PRIMARY KEY' 
        ) Q1 
        ON Q1.TABLE_NAME = PK.TABLE_NAME 
WHERE pk.TABLE_SCHEMA = 'Access'

OPEN tc_cursor   
FETCH NEXT FROM tc_cursor INTO @TABLE_SCHEMA,
												@PRIMARYTABLE, 
												@FOREIGNKEY,
												@RELATEDTABLE,
												@RELATEDKEY
												
WHILE @@FETCH_STATUS = 0   
BEGIN
	SET @SQL_COMMAND = 'ALTER TABLE ' + @TABLE_SCHEMA + '.[' + @PRIMARYTABLE + ']'
	SET @SQL_COMMAND = @SQL_COMMAND + ' ADD FOREIGN KEY(' + @FOREIGNKEY  + ')'
	SET @SQL_COMMAND = @SQL_COMMAND + ' REFERENCES ' + @TABLE_SCHEMA+ '.[' + @RELATEDTABLE + ']'
	SET @SQL_COMMAND = @SQL_COMMAND + '(' +  @RELATEDKEY + ')'
	PRINT @SQL_COMMAND
	PRINT 'GO'
	FETCH NEXT FROM tc_cursor INTO @TABLE_SCHEMA,
													@PRIMARYTABLE, 
													@FOREIGNKEY,
													@RELATEDTABLE,
													@RELATEDKEY
												
END 

PRINT @SQL_COMMAND
PRINT 'GO'
	
CLOSE tc_cursor   
DEALLOCATE tc_cursor

GO


